library simple_billing;

import 'dart:async';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_inapp_purchase/flutter_inapp_purchase.dart';

part 'pop_up_widget.dart';

class SimpleBilling {
  //! Public Vars
  static Set<String> purchasedIds = {};

  //! Private Vars
  static StreamSubscription _purchaseUpdatedSubscription;
  static StreamSubscription _purchasePromotedSubscription;
  static StreamSubscription _purchaseErrorSubscription;
  static void Function(PurchasedItem productId) _onPurchaseUpdated;
  static List<IAPItem> _productsItems = [];
  static List<IAPItem> _subscriptionsItems = [];
  static BuildContext _context;
  static final Set<String> _consumables = {};
  static Widget _popUpWidget;
  // static AndroidValidation _androidValidation;
  // static IosValidation _iosValidation;

  static Future<void> init({
    List<String> productsIds = const [],
    List<String> subscriptionsIds = const [],
    Widget popUpWidget = const _DefaultPopUpWidget(),
    void Function(PurchasedItem productId) onPurchaseUpdated,
    // AndroidValidation androidValidation,
    // IosValidation iosValidation,
  }) async {
    //! Set Private items
    _popUpWidget = popUpWidget;
    _onPurchaseUpdated = onPurchaseUpdated;
    // _androidValidation = androidValidation;
    // _iosValidation = iosValidation;
    //! Init Connection
    await FlutterInappPurchase.instance.initConnection;
    //! Create Listeners
    _purchaseErrorListen();
    _purchaseUpdateListen();
    _purchasePromotedListen();
    //! Get IAPItems
    await _getSubscriptions(subscriptionsIds);
    await _getProducts(productsIds);
  }

  //! Public Functions

  static Future<void> buyProduct(String productId,
      {@required BuildContext context, bool consumable = false}) async {
    if (consumable) {
      _consumables.add(productId);
    }
    _openLoading(context);
    await FlutterInappPurchase.instance.requestPurchase(productId);
  }

  static Future buySubscription(String productId,
      {@required BuildContext context}) async {
    _openLoading(context);
    await FlutterInappPurchase.instance.requestSubscription(productId);
  }

  static Future<void> restorePurchases({@required BuildContext context}) async {
    _openLoading(context);
    final items = await FlutterInappPurchase.instance.getPurchaseHistory();
    items.forEach(_onPurchaseUpdated);
    _closeLoading();
  }

  static String getPrice(String productId) {
    return getProduct(productId)?.price;
  }

  static IAPItem getProduct(String productId) {
    return _getProductById(productId) ?? _getSubscriptionById(productId);
  }

  static bool checkPurchase(String productId) {
    return purchasedIds.contains(productId);
  }

  static bool checkIfHaveAnyPurchase(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a || b);

  static bool checkIfEverythingIsPurchased(List<String> productId) =>
      productId.map((e) => checkPurchase(e)).reduce((a, b) => a && b);

  //! Privates Functions

  /*static Future<Response> _validateReceiptAndroid(PurchasedItem purchased,
      {bool isSubscription = false}) async {
    final Response response =
        await FlutterInappPurchase.instance.validateReceiptAndroid(
      packageName: _androidValidation.packageName,
      accessToken: _androidValidation.accessToken,
      productId: purchased.productId,
      productToken: purchased.purchaseToken,
      isSubscription: isSubscription,
    );
    print('validateReceiptAndroid response = $response');
    return response;
  }

  static Future<Response> _validateReceiptIos(PurchasedItem purchased) async {
    final receiptBody = {
      'receipt-data': purchased.transactionReceipt,
      'password': _iosValidation.password,
    };
    final Response response = await FlutterInappPurchase.instance
        .validateReceiptIos(receiptBody: receiptBody);
    print('validateReceiptIos response = $response');
    return response;
  }*/

  static void _closeLoading() {
    if (_context != null) {
      Navigator.pop(_context);
      _context = null;
    }
  }

  static void _openLoading(BuildContext context) {
    _context = context;
    showDialog(context: context, builder: (context) => _popUpWidget);
  }

  static IAPItem _getProductById(String productId) =>
      _productsItems.firstWhere((e) => e.productId == productId);

  static IAPItem _getSubscriptionById(String productId) =>
      _subscriptionsItems.firstWhere((e) => e.productId == productId);

  static Future<void> _getSubscriptions(List<String> subscriptionsIds) async {
    if (Platform.isAndroid && subscriptionsIds.isNotEmpty) {
      _subscriptionsItems = await FlutterInappPurchase.instance
          .getSubscriptions(subscriptionsIds);
    }
  }

  static Future<void> _getProducts(List<String> productsIds) async {
    if (productsIds.isNotEmpty) {
      _productsItems =
          await FlutterInappPurchase.instance.getProducts(productsIds);
    }
  }

  static void _purchaseUpdateListen() {
    _purchaseUpdatedSubscription = FlutterInappPurchase.purchaseUpdated
        .listen((PurchasedItem purchase) async {
      _closeLoading();
      print('purchase = ${purchase.toString()}');
      _onPurchaseUpdated(purchase);
      if (Platform.isAndroid && _consumables.contains(purchase.productId)) {
        print('consumePurchaseAndroid = ${purchase.productId}');
        await FlutterInappPurchase.instance
            .consumePurchaseAndroid(purchase.purchaseToken);
      }
    });
  }

  static void _purchaseErrorListen() {
    _purchaseErrorSubscription = FlutterInappPurchase.purchaseError
        .listen((PurchaseResult purchaseError) {
      _closeLoading();
      print('purchaseError = ${purchaseError.toString()}');
    });
  }

  static void _purchasePromotedListen() {
    _purchasePromotedSubscription =
        FlutterInappPurchase.purchasePromoted.listen(
      (String onData) {
        print('purchasePromoted onData = $onData');
      },
      onError: () {
        print('purchasePromoted onError');
      },
      onDone: () {
        print('purchasePromoted onDone');
      },
      cancelOnError: false,
    );
  }

  static Future<void> dispose() async {
    await FlutterInappPurchase.instance.endConnection;
    _purchaseUpdatedSubscription?.cancel();
    _purchaseErrorSubscription?.cancel();
    _purchasePromotedSubscription?.cancel();
  }
}

class AndroidValidation {
  AndroidValidation({@required this.packageName, @required this.accessToken});
  final String packageName;
  final String accessToken;
}

class IosValidation {
  IosValidation({@required this.password});
  final String password;
}
